@@ -1,8 +1,13 @@
+#define ECB_NO_THREADS 1
+
 #define EV_COMPAT3 0
 #define EV_USE_SELECT 1
 #define EV_USE_EPOLL 1
 #define EV_USE_KQUEUE 1
 #define EV_USE_PORT 1
+//#define EV_USE_LINUXAIO 1
+//#define EV_USE_IOURING 1
+//#define EV_USE_TIMERFD 0
 #define EV_PREPARE_ENABLE 1
 #define EV_IDLE_ENABLE 1
 #define EV_SIGNAL_ENABLE 1
@@ -13,7 +18,7 @@
 #include <config.h>
 
 #if ENABLE_FRILLS
-# define EV_FEATURES 1+2
+# define EV_FEATURES 1 | 2
 #else
 # define EV_FEATURES 0
 # define NDEBUG
