@@ -73,9 +73,9 @@ struct rxvt_fontset
 
   // must be power-of-two - 1, also has to match RS_fontMask in rxvt.h
 #if USE_256_COLORS
-  enum { fontCount =   7 }; // 4 extra colors bits, 4 fewer fontcount bits
+  enum { fontCount =   7 }; // 2 extra colors bits, 2 fewer fontcount bits
 #else
-  enum { fontCount = 127 };
+  enum { fontCount =  31 };
 #endif
 
   // index of first font in set
@@ -101,6 +101,15 @@ struct rxvt_fontset
     return min<int> ((fontCount << 1) | 1, find_font_idx (unicode));
   }
 
+  // find the font containing ' ' - we always assume this is font 1, as
+  // every font should contain space, and font 1 is our base font.
+  // pango assumes this, so it must be correct!
+  int
+  find_space_font ()
+  {
+    return 1 << 1;
+  }
+
 private:
   rxvt_term *term;
   rxvt_fontprop prop;
